/******************************************************************************
 * @file        MPULCD.c
 * @version     V1.00.02
 * $Revision:   0 $
 * $Date:       2017-05-11 $
 * @brief       MPU LCD Driver source file
 *
 * @note        V1.00.02 - remove MPULCD_Type. Format constant.
 *
 * Copyright (C) 2017 AMICCOM Electronics Corp. All rights reserved.
 *****************************************************************************/

#include "system_A3117M0.h"
#include "MPULCD.h"

/** @addtogroup A3117M0_Driver A3117M0 Driver
    @{
*/
/** @addtogroup MPULCD_Driver MPULCD Driver
    @{
*/

sLCD LcdModule;
uint32_t DMA_RAM_Start;

/**
 *
 * @brief   Write data through MPULCD interface.
 *
 * @param   data            The data will send through MPULCD interface
 * @param   a0              Set A0 signal. 0: A0=0, 1: A0=1
 * @return  none
 */
void MPULCD_Write(uint16_t data, uint8_t a0)
{
    MPULCD->CONFIG |= MPULCD_CONFIG_CMD_Msk;        // command mode. Only send 1 data.
    if(0 == a0)
        MPULCD->CONFIG &= ~MPULCD_CONFIG_A0_Msk;
    else
        MPULCD->CONFIG |= MPULCD_CONFIG_A0_Msk;
    MPULCD->CONFIG &= ~MPULCD_CONFIG_RNW_Msk;       // RnW=0: nRD=1, nWR=0
    MPULCD->CMDDAT = data;
    MPULCD->CONFIG |= MPULCD_CONFIG_ENABLE_Msk;     // send start
    while(0 != (MPULCD->CONFIG & MPULCD_CONFIG_BUSY_Msk))  ;
    MPULCD->CONFIG &= ~MPULCD_CONFIG_ENABLE_Msk;    // send finish
}

/**
 *
 * @brief   Read data through MPULCD interface.
 *
 * @param   a0              Set A0 signal. 0: A0=0, 1: A0=1
 * @return  The data read through MPULCD interface.
 */
uint16_t MPULCD_Read(uint8_t a0)
{
    uint16_t read;

    MPULCD->CONFIG |= MPULCD_CONFIG_CMD_Msk;        // command mode. Only send 1 data.
    if(0 == a0)
        MPULCD->CONFIG &= ~MPULCD_CONFIG_A0_Msk;
    else
        MPULCD->CONFIG |= MPULCD_CONFIG_A0_Msk;
    MPULCD->CONFIG |= MPULCD_CONFIG_RNW_Msk;        // RnW=1: nRD=0, nWR=1
    MPULCD->CONFIG |= MPULCD_CONFIG_ENABLE_Msk;     // send start.
    while(0 != (MPULCD->CONFIG & MPULCD_CONFIG_BUSY_Msk))  ;
    MPULCD->CONFIG &= ~MPULCD_CONFIG_ENABLE_Msk;    // send finish
    read = MPULCD->CMDDAT;                          // read data
    MPULCD->CONFIG &= ~MPULCD_CONFIG_RNW_Msk;
    return read;
}

/**
 *
 * @brief   Enable DMA and wait DMA finish
 *
 * @param   offset              DMA SRAM offset
 * @return  none 
 */
void MPULCD_EnableDMA(uint32_t offset)
{
    //---------------------------------------------------------------------
    // Start DMA function. MCU can't access DMA_RAM in this time
    // Enable DMA
    MPULCD->OFFSET = offset;
    MPULCD->CONFIG &= ~MPULCD_CONFIG_CMD_Msk;       // MPULCD_CONFIG_CMD_Msk = 0. Send MPULCD SRAM
    MPULCD->CONFIG |= MPULCD_CONFIG_A0_Msk;
    MPULCD->CONFIG |= MPULCD_CONFIG_ENABLE_Msk;
    // Wait DMA finish
    while(MPULCD->CONFIG & MPULCD_CONFIG_BUSY_Msk)    ;
    // Clean MPULCD_CONFIG_ENABLE_Msk by manual
    MPULCD->CONFIG &= ~ MPULCD_CONFIG_ENABLE_Msk;   // need clean ENABLE=0 before ENABLE=1
    // DMA finish
    //---------------------------------------------------------------------
}


/*@}*/ /* end of group MPULCD_Driver */
/*@}*/ /* end of group A3117M0_Driver */
