/******************************************************************************
 * @file        RTC.c
 * @version     V1.00.03
 * $Revision:   0 $
 * $Date:       2018-04-16 $
 * @brief       RTC TIMER Driver source file
 *
 * @note        V1.00.02: Modify function for robust & correctness.
 *
 * Copyright (C) 2017 AMICCOM Electronics Corp. All rights reserved.
 *****************************************************************************/

#include "system_A3117M0.h"
#include "Systick_Delay.h"

/** @addtogroup A3117M0_Driver A3117M0 Driver
    @{
*/
/** @addtogroup RTC_Driver Real Time Clock Driver
    @{
*/

/**
 *
 * @brief   Initial RTC
 *
 * @param   clksel                  Clock source select. 0=32KHz, 1=32.768KHz
 * @param   week                    Set WEEK, range=0~6
 * @param   hour                    Set HOUR, range=0~23
 * @param   min                     Set MIN,  range=0~59
 * @param   sec                     Set SEC,  range=0~59
 * @return  none
 */
void RTC_Initial(uint32_t clksel, uint32_t week, uint32_t hour, uint32_t min, uint32_t sec)
{
    RTC->CTRL = 0;
    RTC->COUNT = (((week << RTC_COUNT_WEEK_Pos) & RTC_COUNT_WEEK_Msk) |
                  ((hour << RTC_COUNT_HOUR_Pos) & RTC_COUNT_HOUR_Msk) |
                  ((min << RTC_COUNT_MIN_Pos) & RTC_COUNT_MIN_Msk) |
                  ((sec << RTC_COUNT_SEC_Pos) & RTC_COUNT_SEC_Msk)) ;
    RTC->CTRL = (((clksel << RTC_CTRL_CLKSEL_Pos) & RTC_CTRL_CLKSEL_Msk) |
                   RTC_CTRL_EN_Msk);
}

/**
 *
 * @brief   RTC Clock on, RTC Start counting
 *
 * @return  none
 */
void RTC_Start(void)
{
    RTC->CTRL |= RTC_CTRL_ON_Msk;
}


/**
 *
 * @brief   RTC Clock off, RTC Stop count
 *
 * @return  none
 */
void RTC_Stop(void)
{
    RTC->CTRL &= ~RTC_CTRL_ON_Msk;
}

/**
 *
 * @brief   Modify RTC count value
 *
 * @param   clksel                  Clock source select. 0=32KHz, 1=32.768KHz
 * @param   week                    Set WEEK, range=0~6
 * @param   hour                    Set HOUR, range=0~23
 * @param   min                     Set MIN,  range=0~59
 * @param   sec                     Set SEC,  range=0~59
 * @return  none
 */
void RTC_ModifyCount(uint32_t week, uint32_t hour, uint32_t min, uint32_t sec)
{
    uint32_t control, count;
    control = RTC->CTRL;
    count = (((week << RTC_COUNT_WEEK_Pos) & RTC_COUNT_WEEK_Msk) |
             ((hour << RTC_COUNT_HOUR_Pos) & RTC_COUNT_HOUR_Msk) |
             ((min << RTC_COUNT_MIN_Pos) & RTC_COUNT_MIN_Msk) |
             ((sec << RTC_COUNT_SEC_Pos) & RTC_COUNT_SEC_Msk));

    RTC->CTRL &= ~(RTC_CTRL_EN_Msk);
    RTC->COUNT =  count;
    Delay10us(4);                    // wait 1 RTC clock (1/32K = 31.25us)
    RTC->CTRL = control;
}

/**
 *
* @brief   Set Alarm. The Alarm interrupt will occure when RTC->CTRL[hour:min] == RTC->ALARM[hour:min]
 *
 * @param   hour                    Set HOUR, range=0~23
 * @param   min                     Set MIN,  range=0~59
 * @return  none
 */
void RTC_SetAlarm(uint32_t hour, uint32_t min)
{
    RTC->ALARM = (((hour << RTC_ALARM_HOUR_Pos) & RTC_ALARM_HOUR_Msk) |
                  ((min << RTC_ALARM_MIN_Pos) & RTC_ALARM_MIN_Msk));
}

/**
 *
 * @brief   Set Manual Count
 *
 * @param   en                      Menaul Count Enable. 0=disable, 1=enable
 * @param   count                   Menaul Count set
 * @return  none
 */
void RTC_ManualCount_Set(uint32_t en, uint32_t count)
{
    RTC->MANUALCOUNT = (count << RTC_MANUALCOUNT_COUNT_Pos) & RTC_MANUALCOUNT_COUNT_Msk;
    if(0 == en)
    {
        RTC->CTRL &= ~RTC_CTRL_MANUALCOUNTEN_Msk;
    }
    else
    {
        RTC->CTRL |= RTC_CTRL_MANUALCOUNTEN_Msk;
    }
}

/**
 *
 * @brief   Enable RTC interrupt
 *
 * @param   alarm                   Alarm interrupt.        0=disable, 1=enable.
 * @param   sec                     Sec interrupt.          0=disable, 1=enable.
 * @param   half_sec                Half sec interupt.      0=disable, 1=enable.
 * @param   ten_ms                  Ten ms interrupt.       0=disable, 1=enable.
 * @param   manual                  Menaul Count interrupt. 0=disable, 1=enable.
 * @return  none
 */
void RTC_Interrupt_Enable(uint32_t alarm, uint32_t sec, uint32_t half_sec, uint32_t ten_ms, uint32_t manual_count)
{
    uint32_t control = RTC->CTRL & (~(RTC_CTRL_MANUALCOUNTIE_Msk | RTC_CTRL_TMSIE_Msk | RTC_CTRL_HSECIE_Msk | RTC_CTRL_SECIE_Msk | RTC_CTRL_ALARMIE_Msk));
    control |= (((manual_count << RTC_CTRL_MANUALCOUNTIE_Pos) & RTC_CTRL_MANUALCOUNTIE_Msk) |
                ((ten_ms << RTC_CTRL_TMSIE_Pos) & RTC_CTRL_TMSIE_Msk) |
                ((half_sec << RTC_CTRL_HSECIE_Pos) & RTC_CTRL_HSECIE_Msk) |
                ((sec << RTC_CTRL_SECIE_Pos) & RTC_CTRL_SECIE_Msk) |
                ((alarm << RTC_CTRL_ALARMIE_Pos) & RTC_CTRL_ALARMIE_Msk));
    RTC->CTRL = control;
}


/*@}*/ /* end of group RTC_Driver */
/*@}*/ /* end of group A3117M0_Driver */
